//
// snappy-remote - Tool to download work with remote snappy instances.
//
// Copyright (c) 2014 Canonical Ltd.
//
// Written by Sergio Schvezov <sergio.schvezov@canonical.com>
//
package main

import (
	"errors"
	"fmt"
	"net/url"
	"os"

	"github.com/jessevdk/go-flags"
)

// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

var parser = flags.NewParser(&globalArgs, flags.Default)

type arguments struct {
	Url    string `long:"url" description:"the url of the remote system" required:"true"`
	PubKey string `long:"pub-key" description:"the public key to use to authenticate"`
	remote Remote
}

var globalArgs arguments

type Remote interface {
	Run(command string) error
	Copy(src, dst string) error
	NeedsKeys() bool
	SetupKey(keyPath string) error
}

func main() {
	if _, err := parser.Parse(); err != nil {
		os.Exit(1)
	}
}

func (a arguments) setRemote() error {
	u, err := url.Parse(globalArgs.Url)
	if err != nil {
		return err
	}

	switch u.Scheme {
	case "ssh":
		globalArgs.remote, err = newSshRemote(u.Host)
		if err != nil {
			return err
		}
	default:
		return errors.New("unsupported url")
	}

	return nil
}

const noKeyWarn = "WARNING, trying automatic key setup, if it fails " +
	"use --pub-key [path-to-key] or input your password on " +
	"every request."

func setup() error {
	if err := globalArgs.setRemote(); err != nil {
		return err
	}

	if globalArgs.remote.NeedsKeys() {
		fmt.Println(noKeyWarn)
		if err := globalArgs.remote.SetupKey(globalArgs.PubKey); err != nil {
			fmt.Println("Failed to setup keys:", err)
		}
	}

	return nil
}
