//
// snappy-remote - Tool to download work with remote snappy instances.
//
// Copyright (c) 2014 Canonical Ltd.
//
// Written by Sergio Schvezov <sergio.schvezov@canonical.com>
//
package main

import (
	"errors"
	"fmt"
	"io/ioutil"
	"net"
	"os"
	"os/exec"
	"path/filepath"
	"strings"
	"time"
)

// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

type sshRemote struct {
	host string
	port string
	Remote
}

func newSshRemote(hostport string) (*sshRemote, error) {
	host, port, err := net.SplitHostPort(hostport)
	if err != nil {
		if netErr, ok := err.(*net.AddrError); ok {
			if netErr.Err == "missing port in address" {
				host = hostport
				port = "22"
			} else {
				return nil, err
			}
		} else {
			return nil, err
		}
	}

	remote := &sshRemote{
		host: host,
		port: port,
	}

	return remote, nil
}

func getAuthorizedSshKey() (string, error) {
	sshDir := os.ExpandEnv("$HOME/.ssh")

	fis, err := ioutil.ReadDir(sshDir)
	if err != nil {
		return "", fmt.Errorf("%s: no pub ssh key found, run ssh-keygen first", err)
	}

	var preferredPubKey string
	var latestModTime time.Time
	for i := range fis {
		file := fis[i].Name()
		if strings.HasSuffix(file, ".pub") && !strings.HasSuffix(file, "cert.pub") {
			fileMod := fis[i].ModTime()

			if fileMod.After(latestModTime) {
				latestModTime = fileMod
				preferredPubKey = file
			}
		}
	}

	if preferredPubKey == "" {
		return "", errors.New("no pub ssh key found, run ssh-keygen first")
	}

	pubKey, err := ioutil.ReadFile(filepath.Join(sshDir, preferredPubKey))

	return string(pubKey), err
}

func (remote *sshRemote) SetupKey(keyPath string) error {
	var publicKey string

	if keyPath == "" {
		key, err := getAuthorizedSshKey()
		if err != nil {
			return err
		}
		publicKey = key
	} else {
		key, err := ioutil.ReadFile(keyPath)
		if err != nil {
			return err
		}
		publicKey = string(key)
	}

	command := fmt.Sprintf("echo \"%s\" >> ~/.ssh/authorized_keys", publicKey)

	if err := remote.Run(command); err != nil {
		return fmt.Errorf("failed to setup keys: %s", err)
	}

	return nil
}

func (remote *sshRemote) NeedsKeys() bool {
	if err := exec.Command("ssh", "-o", "PasswordAuthentication=no", fmt.Sprintf("ubuntu@%s", remote.host), "-p", remote.port, "ls").Run(); err != nil {
		return true
	} else {
		return false
	}
}

func (remote *sshRemote) Run(command string) error {
	out, err := exec.Command("ssh", fmt.Sprintf("ubuntu@%s", remote.host), "-p", remote.port, command).CombinedOutput()
	if err != nil {
		return fmt.Errorf("issues while running ssh command: %s", out)
	}

	fmt.Println(string(out))

	return nil
}

func (remote *sshRemote) Copy(src, dst string) error {
	out, err := exec.Command("scp", "-P", remote.port, src, fmt.Sprintf("ubuntu@%s:%s", remote.host, dst)).CombinedOutput()
	if err != nil {
		return fmt.Errorf("issues while running ssh command: %s", out)
	}

	return nil
}
